package com.example.demo.config;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.bson.types.ObjectId;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.data.spel.spi.EvaluationContextExtension;
import org.springframework.data.spel.spi.Function;
import org.springframework.util.ReflectionUtils;

public class GlobalEvaluation implements EvaluationContextExtension {
	private static final Logger logger = LoggerFactory.getLogger(GlobalEvaluation.class);

	@Autowired
	Environment env;

	@Override
	public String getExtensionId() {
		return "global";
	}

	@Override
	public Map<String, Function> getFunctions() {

		Map<String, Function> functions = new HashMap<>();

		Method method = ReflectionUtils.findMethod(GlobalEvaluation.class, "getUser", String.class);

		Function function = new Function(method, this);

		Method method2 = ReflectionUtils.findMethod(GlobalEvaluation.class, "getOid", Object.class);

		Function function2 = new Function(method2, this);

		functions.put("getProperty", function);
		functions.put("getOid", function2);

		return functions;
	}

	public Object getOid(Object param) {
		if (param instanceof String) {
			return new ObjectId((String) param);
		}
		if (param instanceof List) {
			List<String> ids = (List<String>) param;
			return ids.stream().map(i -> new ObjectId(i)).collect(Collectors.toList());
		}
		logger.error("Cannot convert {} to an OID", param);
		return null;
	}

	public String getProperty(String param) {
		return env.getProperty(param);
	}

}
